unit Unit1;

interface

uses
  LibGfl, Jpeg,
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Vcl.ExtCtrls, Vcl.ExtDlgs;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    Label1: TLabel;
    btnSelect: TButton;
    OpenPictureDialog1: TOpenPictureDialog;
    Panel2: TPanel;
    Image1: TImage;
    Panel3: TPanel;
    Splitter1: TSplitter;
    Image2: TImage;
    procedure btnSelectClick(Sender: TObject);
  private
    { Private declarations }
    procedure ConvertImage(ImageFilePathName, sFilePathName: string; const NewImageLength: cardinal; var IsImage: boolean);
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}


procedure TForm1.btnSelectClick(Sender: TObject);
var
  IsImage: boolean;
  s, sFilePath, sFilePathName, ImageFilePathName, ImageFileName: string;
begin
  s := GetCurrentDir;
  try
    OpenPictureDialog1.Filter := 'Image files|*.JP*;*.WMF;*.EMF;*.BMP;*.GIF;*.PNG;*.Ti*;*.CAM;*.KDC|All files|*.*';

    if OpenPictureDialog1.Execute() then
    begin
      ImageFilePathName := OpenPictureDialog1.FileName;
      ImageFileName := ExtractFileName(ImageFilePathName);

      sFilePath := s + '\Resized';
      if not DirectoryExists(sFilePath) then
      begin
        if not CreateDir(sFilePath ) then
          raise Exception.Create('Can''t create sFilePath');
      end;


      sFilePathName := sFilePath + '\' + ImageFileName;
      ConvertImage(ImageFilePathName, sFilePathName, 300, IsImage);

      if IsImage then
      begin
        Image1.Picture.LoadFromFile(ImageFilePathName);
        Image2.Picture.LoadFromFile(sFilePathName);
      end
      else
        raise Exception.Create('File selected is not an image');

      Image1.Visible := true;
    end;
  finally
    ChDir(s);
  end;
end;

procedure TForm1.ConvertImage(ImageFilePathName, sFilePathName: string; const NewImageLength: cardinal; var IsImage: boolean);   {based on GFL SDK version 3.11}
var
  j: integer;
  width, new_width, height, new_height: cardinal;
  lp: TGFL_LOAD_PARAMS;
  sp: TGFL_SAVE_PARAMS;
  gfl_bmp: PGFL_BITMAP;
  finfo: TGFL_FILE_INFORMATION;

  e,f: GFL_ERROR;
  format: string;

  ConvertError: boolean;
begin
  gflEnableLZW(GFL_TRUE);
  ConvertError := false;
  IsImage := false;

  if ImageFilePathName <> '' then
  begin
    gflGetDefaultLoadParams(lp);
    e := gflLoadBitmapW(PWideChar(ImageFilePathName), gfl_bmp, lp, finfo);

    if (e <> gfl_no_error) then
    begin
      MessageDlg('File not readable: ' + string(gflGetErrorString(e)), mtError, [mbOK], 0);
      exit;
      ConvertError := true;
    end;

    if not ConvertError then
    begin
      IsImage := true;

      width := finfo.Width;
      height := finfo.Height;

      if width > height then
      begin
        new_width := NewImageLength;
        new_height := Round((new_width / width) * height);
      end
      else
      begin
        new_height := NewImageLength;
        new_width := Round((new_height / height) * width);
      end;

      gflResize(gfl_bmp, nil, new_width, new_height, GFL_RESIZE_BILINEAR, 0);

      j := 0; {jpg}

      format := gflGetDefaultFormatSuffixByIndex(j);
      gflGetDefaultSaveParams(sp);
      sp.FormatIndex := j;
      if j = 0 then
      begin
        sp.Quality := 100;
        sp.Flags := GFL_SAVE_ANYWAY;
      end;

      f := gflSaveBitmapW(PWideChar(sFilePathName),gfl_bmp, sp);

      if (f <> gfl_no_error) then
      begin
        ConvertError := true;
      end;

      gflFreeBitmap(gfl_bmp);
    end;
  end;
end;

end.
